#ifndef _LIB_SVGA_H_
#define _LIB_SVGA_H_

#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
#include <vgamouse.h>

#include "lib_image.h"
#include "lib_font.h"


#ifdef __DJGPP__
#  define NR_KEYS       128
#  define CURSOR_UP     SCANCODE_CURSORUP
#  define CURSOR_DOWN   SCANCODE_CURSORDOWN
#else
#  define CURSOR_UP     SCANCODE_CURSORBLOCKUP
#  define CURSOR_DOWN   SCANCODE_CURSORBLOCKDOWN
#endif

#define in_rectangle(x,y,x1,y1,w,h)   (((x) > (x1)) && ((x) < ((x1) + (w))) && ((y) > (y1)) && ((y) < ((y1) + (h))))
#define BPP_SHIFT                     ((svga_flags & SVGA_8BPP_PALETTE) ? 0 : 2)
#define BPP_MAXVALUE                  ((svga_flags & SVGA_8BPP_PALETTE) ? 255 : 63)

/***
 * Some important VGA registers
 */
#define STATUS1                     (vgaIOBase + 0x0A)
#define ATR_CTL_INDEX_DATA_SWITCH   STATUS1
#define ATR_CTL_INDEX               0x3C0
#define ATR_CTL_DATA_W              ATR_CTL_INDEX
#define MIS_R                       0x3CC    /* Misc Output Read Register                        */
#define SEQ_I                       0x3C4    /* Sequencer Index                                  */
#define SEQ_D                       0x3C5    /* Sequencer Data Register                          */
#define ATT_IW                      0x3C0    /* Attribute Controller Index & Data Write Register */
#define IS1_RC                      0x3DA    /* Input Status Register 1 - color emulation        */

enum SVGA_flags
{
   SVGA_OPEN             = 0x0001,
   SVGA_8BPP_PALETTE     = 0x0002,
   SVGA_DOUBLE_BUFFERING = 0x0004,
   SVGA_TRIPLE_BUFFERING = 0x0008,
};


extern struct FONT_DATA svga_default_font;


#if defined (__STDC__) || defined (__cplusplus) || defined (NeedFunctionPrototypes)
#  define __PARMS(s) s
#else
#  define __PARMS(s) ()
#endif

#ifdef DEBUG
void               svga_show_keytable  __PARMS((void));
void               svga_show_palette   __PARMS((short x, short y, short block_size));
#endif

short              svga_init           __PARMS((short mode, short lock_flag));
short              svga_virtual_init   __PARMS((short mode, short buffering_flag));
void               svga_close          __PARMS((void));
palette_t         *svga_palette        __PARMS((void));
GraphicsContext   *svga_gc             __PARMS((void));
void               svga_get_palette    __PARMS((palette_t *palette_data));
void               svga_set_palette    __PARMS((palette_t *palette_data, short copy_flag));
void               svga_palette_fade   __PARMS((short direction));
void               svga_set_border     __PARMS((unchar color));
void               svga_screen_on      __PARMS((void));
void               svga_screen_off     __PARMS((void));
void               svga_clean          __PARMS((void));
short              svga_show_image     __PARMS((char *filename, short x, short y, short width, short height, short x1, short y1));
void               svga_draw_rectangle __PARMS((short x, short y, short width, short height, short color));
short              svga_save_snapshot  __PARMS((void));
void               svga_copyscreen     __PARMS((short wait_retrace_flag));
void               svga_printf         __PARMS((short x, short y, short color, struct FONT_DATA *font, char *format, ...));

#undef __PARMS
#endif /* _LIB_SVGA_H_ */
